/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the plugins of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QACCESSIBLEDECLARATIVEITEM_H
#define QACCESSIBLEDECLARATIVEITEM_H

#include "complexwidgets.h"
#include "qaccessibledeclarativeimplementation.h"
#include <QtDeclarative/qdeclarativeview.h>
#include <QtDeclarative/qdeclarativeitem.h>
#include <QtDeclarative/qdeclarativeproperty.h>
#include <private/qdeclarativeaccessible_p.h>


QT_BEGIN_NAMESPACE

#ifndef QT_NO_ACCESSIBILITY

/*
    -- Declarative Accessibility Overview. --

    QML accessibility is implemented using two class hierarchies and two "view" interface classes.

    * The interface hierarchy.
    This hierarchy implements QAccessibleInterfaces for the various QML interfaces.
    The root of the hierarchy is QAccessibleDeclarativeObject - subclass as necessary
    to implement QML items.

    * The implementation hierarchy.
    This hierarchy abstracts the implementation differences between Qt Quick1 and Qt Quick2.
    The base class is QAccessibleDeclarativeImplementation, with two subclasses:
    QAccessibleGraphicsViewImplementation
    QAccessibleSceneGraphImplementation

    The two hierarchies are connected: each QAccessibleDeclarativeObject owns a
    QAccessibleDeclarativeImplementation. (This is the Bridge pattern, for the
    design pattern inclined.)

    * View interface classes.

    These are the root of the QML accessible tree and connects it to the widget hierarchy.

    QAccessbileDeclarativeView is the root for the QGraphicsView implementation
    QAccessbileSceneGraphView is the root for the SceneGraph implementation


*/
class QAccessibleDeclarativeObject: public QAccessibleObjectEx
{
    Q_ACCESSIBLE_OBJECT
public:
    // Use this factory method to instantiate objects. This will create an accessible object of the
    // correct type depending the accessibleRole attribute. Implementation should be one of
    // graphicsview or scenegraph.
    static QAccessibleDeclarativeObject *instantiateObject(QObject *item, QAccessibleDeclarativeImplementation *implementation);

    ~QAccessibleDeclarativeObject();

    QRect rect(int child) const;
    int childCount() const;
    int navigate(RelationFlag rel, int entry, QAccessibleInterface **target) const;
    int indexOfChild(const QAccessibleInterface*) const;
    QFlags<QAccessible::RelationFlag> relationTo(int, const QAccessibleInterface*, int) const;
    int childAt(int, int) const;
    QString text(QAccessible::Text, int) const;
    QAccessible::Role role(int) const;
    QFlags<QAccessible::StateFlag> state(int) const;
    bool doAction(int action, int child, const QVariantList &params);
    QVariant invokeMethodEx(QAccessible::Method, int, const QVariantList&);
protected:
    // For sublcasses, use instantiateObject factory methotd outside the class.
    QAccessibleDeclarativeObject(QObject *object, QAccessibleDeclarativeImplementation *implementation, QAccessible::Role role);

private:
    bool hasChildren() const;

private:
    QAccessibleDeclarativeImplementation *m_implementation;
    QAccessible::Role m_role;
};

class QAccessibleDeclarativeTextObject
        : public QAccessibleDeclarativeObject, public QAccessibleTextInterface
{
    Q_ACCESSIBLE_OBJECT
public:
    virtual void addSelection(int startOffset, int endOffset);
    virtual QString attributes(int offset, int *startOffset, int *endOffset);
    virtual int cursorPosition();
    virtual QRect characterRect(int offset, QAccessible2::CoordinateType coordType);
    virtual int selectionCount();
    virtual int offsetAtPoint(const QPoint &point, QAccessible2::CoordinateType coordType);
    virtual void selection(int selectionIndex, int *startOffset, int *endOffset);
    virtual QString text(int startOffset, int endOffset);
    virtual QString textBeforeOffset (int offset, QAccessible2::BoundaryType boundaryType,
                              int *startOffset, int *endOffset);
    virtual QString textAfterOffset(int offset, QAccessible2::BoundaryType boundaryType,
                            int *startOffset, int *endOffset);
    virtual QString textAtOffset(int offset, QAccessible2::BoundaryType boundaryType,
                         int *startOffset, int *endOffset);
    virtual void removeSelection(int selectionIndex);
    virtual void setCursorPosition(int position);
    virtual void setSelection(int selectionIndex, int startOffset, int endOffset);
    virtual int characterCount();
    virtual void scrollToSubstring(int startIndex, int endIndex);

    // add invoke extension
    QFlags<QAccessible::StateFlag> state(int) const;

    virtual void setText(Text t, int child, const QString &text);

protected:
    QAccessibleDeclarativeTextObject(QObject *object, QAccessibleDeclarativeImplementation *implementation, QAccessible::Role role);
    QString text() const;
friend class QAccessibleDeclarativeObject;
};

class QAccessibleDeclarativeEditableTextObject
        : public QAccessibleDeclarativeTextObject, public QAccessibleSimpleEditableTextInterface
{

protected:
    QAccessibleDeclarativeEditableTextObject(QObject *object, QAccessibleDeclarativeImplementation *implementation, QAccessible::Role role)
        : QAccessibleDeclarativeTextObject(object, implementation, role), QAccessibleSimpleEditableTextInterface(this)
    {}

friend class QAccessibleDeclarativeObject;
};

#endif // QT_NO_ACCESSIBILITY

QT_END_NAMESPACE

#endif // QACCESSIBLEDECLARATIVEITEM_H
