/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the plugins of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qaccessibledeclarativeview.h"
#include "qaccessibledeclarativeobject.h"
#include "qaccessiblegraphicsviewimplementation.h"

#ifndef QT_NO_ACCESSIBILITY

QT_BEGIN_NAMESPACE

QAccessibleDeclarativeView::QAccessibleDeclarativeView(QWidget *widget)
    :QAccessibleAbstractScrollArea(widget)
{
    m_view = static_cast<QDeclarativeView *>(widget);
}

int QAccessibleDeclarativeView::childCount() const
{
    return 1;
}

int QAccessibleDeclarativeView::navigate(RelationFlag rel, int entry, QAccessibleInterface **target) const
{
    //qDebug() << "navigate" << rel << entry << "root" << (void*)m_view->accessibleRootItem();
    QDeclarativeItem *declarativeRoot = m_view->accessibleRootItem();

    if (rel == QAccessible::Child) {
        if (entry != 1 || !declarativeRoot) {
            *target = 0;
            return -1;
        }
        *target = QAccessibleGraphicsViewImplementation::instantiateObject(declarativeRoot, m_view);
        return 0;
    }

    return QAccessibleAbstractScrollArea::navigate(rel, entry, target);
}

QString QAccessibleDeclarativeView::text(QAccessible::Text textType, int child) const
{
    if (textType == QAccessible::DebugDescription) {
        return QLatin1String("QDeclarativeView");
    }
    return QAccessibleAbstractScrollArea::text(textType, child);
}

int QAccessibleDeclarativeView::childAt(int x, int y) const
{
    return 1; // return the top-level QML item
}

int QAccessibleDeclarativeView::indexOfChild(const QAccessibleInterface *iface) const
{
    if (iface) {
        QDeclarativeItem *declarativeRoot = m_view->accessibleRootItem();
        if (declarativeRoot == iface->object())
            return 1;
    }
    return -1;

}

QT_END_NAMESPACE

#endif // QT_NO_ACCESSIBILITY
